//
//  PPMElasticTabView.m
//  PPMElasticTabView
//
//  Created by ppm on 07/08/12.
//  Copyright 2007 ppm. All rights reserved.
//

#import "PPMElasticTabView.h"


@implementation PPMElasticTabView

- (void)awakeFromNib
{
	[self expandTabViewItem:[self selectedTabViewItem] animate:NO];
}

- (void)selectTabViewItem:(NSTabViewItem *)tabViewItem
{
	[self expandTabViewItem:tabViewItem animate:YES];
}

- (void)expandTabViewItem:(NSTabViewItem *)tabViewItem animate:(BOOL)performAnimation {

	[self setHidden:YES];

	//いわゆる本来の「コンテントビュー」
	NSView *tabContentView = [tabViewItem view];
	
	// ↑その直下の（最初の）サブビューを、コンテントビュー的に扱う（つまり、本来置きたかったボタンやフィールドなどは、このビューのサブビューとして配置する）。
	// このビューの大きさに合わせて（強制的に）タブビューが伸縮するので、Interface Builder 上では、適当なビューを適当な位置、大きさで配置しておけばよい。
	NSView *myTabContentView = [[tabContentView subviews] objectAtIndex:0];
	
	// タブビューを伸縮したとき、その中身のサブビューをオートリサイズされると厄介なので、
	// オートリサイズ設定を保存して、いったんオートリサイズをオフにする。
	unsigned oldContentAutoResizingMask = [myTabContentView autoresizingMask];
	[myTabContentView setAutoresizingMask:NSViewNotSizable];
	
	// 引数で渡ってきた tabViewItem が、まだ選択されていないものであったら、ここで選択する。
	// -selectTabViewItem:tabViewItem と組み合わせて使うためなのですが、ちょっと微妙？
	if ( tabViewItem != [self selectedTabViewItem] ) [super selectTabViewItem:tabViewItem];
	
	// 上記の「疑似」コンテントビューを、強制的に原点に移動（左下）
	[myTabContentView setFrameOrigin:NSZeroPoint];

	// （※１）「余っている幅」を引く（？）。要するに：本来のコンテントビューのサイズ ＝ 疑似コンテントビューのサイズ、とする。
	NSRect newFrame = [self frame];
	float dW = [tabContentView frame].size.width - [myTabContentView frame].size.width;
	float dH = [tabContentView frame].size.height - [myTabContentView frame].size.height;
	
	newFrame.size.width -= dW;
	newFrame.size.height -= dH;
	
	[self setFrame:newFrame];
	
	[myTabContentView setAutoresizingMask:oldContentAutoResizingMask]; 
	
	NSRect newWindowFrame = [[self window] frame];
	unsigned oldAutoResizingMask = [self autoresizingMask];
	
	// ※１で伸縮した分だけ、ウィンドウも伸縮させる。
	newWindowFrame.origin.x += rint(dW / 2); //こうすると、タブの位置が変化しないので、ちょっとユーザに優しい。（そもそもウィンドウの横幅はあまり変化させない方がいいと思いますが…）
	newWindowFrame.size.width -= dW;
	newWindowFrame.origin.y += dH;
	newWindowFrame.size.height -= dH;
	
	[self setAutoresizingMask:NSViewNotSizable];
	[[self window] setFrame:newWindowFrame display:YES animate:performAnimation];
	[self setAutoresizingMask:oldAutoResizingMask];
	
	[self setHidden:NO];
	
}

@end
